@@ -37,6 +37,7 @@ from pilot.util.common import should_abort, was_pilot_killed
 from pilot.util.constants import PILOT_MULTIJOB_START_TIME, PILOT_PRE_GETJOB, PILOT_POST_GETJOB, PILOT_KILL_SIGNAL, LOG_TRANSFER_NOT_DONE, \
     LOG_TRANSFER_IN_PROGRESS, LOG_TRANSFER_DONE, LOG_TRANSFER_FAILED, SERVER_UPDATE_TROUBLE, SERVER_UPDATE_FINAL, \
     SERVER_UPDATE_UPDATING, SERVER_UPDATE_NOT_DONE
+from pilot.util.container import execute
 from pilot.util.filehandling import get_files, tail, is_json, copy, remove, read_file, write_json, establish_logging, write_file
 from pilot.util.harvester import request_new_jobs, remove_job_request_file, parse_job_definition_file, \
     is_harvester_mode, get_worker_attributes_file, publish_job_report, publish_work_report, get_event_status_file, \
@@ -58,7 +59,7 @@ errors = ErrorCodes()
 
 def control(queues, traces, args):
     """
-    (add description)
+    Main function of job control.
 
     :param queues: internal queues for job handling.
     :param traces: tuple containing internal pilot states.
@@ -281,22 +282,28 @@ def send_state(job, args, state, xml=None, metadata=None):  # noqa: C901
             # store the file in the main workdir
             path = os.path.join(os.environ.get('PILOT_HOME'), config.Pilot.heartbeat_message)
             if write_json(path, data):
+                log.debug('heartbeat dictionary: %s' % data)
                 log.debug('wrote heartbeat to file %s' % path)
                 return True
             else:
                 return False
 
-    #if state == 'running':
-    #    log.info('skipping reporting running for now')
-    #    return True
-
     try:
         # get the URL for the PanDA server from pilot options or from config
         pandaserver = get_panda_server(args.url, args.port)
 
         if config.Pilot.pandajob == 'real':
             time_before = int(time.time())
-            res = https.request('{pandaserver}/server/panda/updateJob'.format(pandaserver=pandaserver), data=data)
+            max_attempts = 3
+            attempt = 0
+            done = False
+            while attempt < max_attempts and not done:
+                log.info('job update attempt %d/%d' % (attempt + 1, max_attempts))
+                res = https.request('{pandaserver}/server/panda/updateJob'.format(pandaserver=pandaserver), data=data)
+                if res is not None:
+                    done = True
+                attempt += 1
+
             time_after = int(time.time())
             log.info('server updateJob request completed in %ds for job %s' % (time_after - time_before, job.jobid))
             log.info("server responded with: res = %s" % str(res))
@@ -780,6 +787,7 @@ def store_jobid(jobid, init_dir):
 
     try:
         path = os.path.join(os.path.join(init_dir, 'pilot2'), config.Pilot.jobid_file)
+        path = path.replace('pilot2/pilot2', 'pilot2')  # dirty fix for bad paths
         mode = 'a' if os.path.exists(path) else 'w'
         logger.debug('path=%s  mode=%s' % (path, mode))
         write_file(path, "%s\n" % str(jobid), mode=mode, mute=False)
@@ -1518,8 +1526,8 @@ def create_job(dispatcher_response, queue):
     logger.info('received job: %s (sleep until the job has finished)' % job.jobid)
     logger.info('job details: \n%s' % job)
 
-    # payload environment wants the PandaID to be set, also used below
-    os.environ['PandaID'] = job.jobid
+    # payload environment wants the PANDAID to be set, also used below
+    os.environ['PANDAID'] = job.jobid
 
     return job
 
@@ -1543,6 +1551,11 @@ def has_job_completed(queues, args):
         log = get_logger(job.jobid, logger)
 
         make_job_report(job)
+        cmd = 'ls -lF %s' % os.environ.get('PILOT_HOME')
+        log.debug('%s:\n' % cmd)
+        ec, stdout, stderr = execute(cmd)
+        log.debug(stdout)
+
         queue_report(queues)
         job.reset_errors()
         log.info("job %s has completed (purged errors)" % job.jobid)
@@ -1554,8 +1567,6 @@ def has_job_completed(queues, args):
 
         return True
 
-    #jobid = os.environ.get('PandaID')
-
     # is there anything in the finished_jobs queue?
     #finished_queue_snapshot = list(queues.finished_jobs.queue)
     #peek = [obj for obj in finished_queue_snapshot if jobid == obj.jobid]
@@ -1908,6 +1919,69 @@ def get_heartbeat_period(debug=False):
         return 1800
 
 
+def check_for_abort_job(args, caller=''):
+    """
+    Check if args.abort_job.is_set().
+    This flag should
+
+    :param args: Pilot arguments (e.g. containing queue name, queuedata dictionary, etc).
+    :param caller: function name of caller (string).
+    :return: Boolean, True if args_job.is_set()
+    """
+    abort_job = False
+    if args.abort_job.is_set():
+        logger.warning('%s detected an abort_job request (signal=%s)' % (caller, args.signal))
+        logger.warning('in case pilot is running more than one job, all jobs will be aborted')
+        abort_job = True
+
+    return abort_job
+
+
+def interceptor(queues, traces, args):
+    """
+    MOVE THIS TO INTERCEPTOR.PY; TEMPLATE FOR THREADS
+
+    :param queues: internal queues for job handling.
+    :param traces: tuple containing internal pilot states.
+    :param args: Pilot arguments (e.g. containing queue name, queuedata dictionary, etc).
+    :return:
+    """
+
+    # overall loop counter (ignoring the fact that more than one job may be running)
+    n = 0
+    while not args.graceful_stop.is_set():
+        time.sleep(0.1)
+
+        # abort in case graceful_stop has been set, and less than 30 s has passed since MAXTIME was reached (if set)
+        # (abort at the end of the loop)
+        abort = should_abort(args, label='job:interceptor')
+
+        # check for any abort_job requests
+        abort_job = check_for_abort_job(args, caller='interceptor')
+        if not abort_job:
+            # peek at the jobs in the validated_jobs queue and send the running ones to the heartbeat function
+            jobs = queues.monitored_payloads.queue
+            if jobs:
+                for i in range(len(jobs)):
+
+                    logger.info('interceptor loop %d: looking for communication file' % n)
+            time.sleep(30)
+
+        n += 1
+
+        if abort or abort_job:
+            break
+
+    # proceed to set the job_aborted flag?
+    if threads_aborted():
+        logger.debug('will proceed to set job_aborted')
+        args.job_aborted.set()
+    else:
+        logger.debug('will not set job_aborted yet')
+
+    logger.debug('[job] interceptor thread has finished')
+
+
 def job_monitor(queues, traces, args):  # noqa: C901
     """
     Monitoring of job parameters.
@@ -1943,30 +2017,25 @@ def job_monitor(queues, traces, args):  # noqa: C901
             logger.warning('job monitor received an abort command')
 
         # check for any abort_job requests
-        abort_job = False
-        if args.abort_job.is_set():
-            logger.warning('job monitor detected an abort_job request (signal=%s)' % args.signal)
-            logger.warning('in case pilot is running more than one job, all jobs will be aborted')
-            abort_job = True
-        elif not queues.current_data_in.empty():
-            # make sure to send heartbeat regularly if stage-in takes a long time
-            jobs = queues.current_data_in.queue
-            if jobs:
-                for i in range(len(jobs)):
-                    # send heartbeat if it is time (note that the heartbeat function might update the job object, e.g.
-                    # by turning on debug mode, ie we need to get the heartbeat period in case it has changed)
-                    update_time = send_heartbeat_if_time(jobs[i], args, update_time)
+        abort_job = check_for_abort_job(args, caller='job monitor')
+        if not abort_job:
+            if not queues.current_data_in.empty():
+                # make sure to send heartbeat regularly if stage-in takes a long time
+                jobs = queues.current_data_in.queue
+                if jobs:
+                    for i in range(len(jobs)):
+                        # send heartbeat if it is time (note that the heartbeat function might update the job object, e.g.
+                        # by turning on debug mode, ie we need to get the heartbeat period in case it has changed)
+                        update_time = send_heartbeat_if_time(jobs[i], args, update_time)
 
+                    # sleep for a while if stage-in has not completed
+                    time.sleep(1)
+                    continue
+            elif queues.finished_data_in.empty():
                 # sleep for a while if stage-in has not completed
                 time.sleep(1)
                 continue
-        elif queues.finished_data_in.empty():
-            # sleep for a while if stage-in has not completed
-            time.sleep(1)
-            continue
 
-        # wait a minute unless we are to abort
-        if not abort_job:
             time.sleep(60)
 
         # peek at the jobs in the validated_jobs queue and send the running ones to the heartbeat function
